      subroutine RAP_all (mat, rbuf, ibuf)
c ======================================================================
c
c        programmer     Jos van Kan
c
c       version 1.2    16-05-97
c         repairs bug in dimension calculation of intermediate
c         coarse grid matrix
c       version 1.1    10-04-1997
c         utilizes module mgpreset instead of common block cmgconst
c       version 1.0    27-03-1997
c
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c
c **********************************************************************
c
c         MODULES USED
c
      use mgpreset
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   calculates RAP operator on all levels.
c   mklvlinf must have been called previously.
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      double precision mat (*), rbuf (*)
      integer ibuf (*)

c     ibuf      i    Integer array of dimension 300 that contains the
c                    administration of this particular multi grid block.
c     mat       i    The nx x ny x 9 coefficient matrix of the
c                    discrete Laplacian
c     rbuf      o    Buffer array used to store all coarse size info
c                    and as scratch space. The actaul bufferspace
c                    needed can be calculated using subr autosize.
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ilevel

c     ilevel    Levelcounter (multi grid levels)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     FREESCRS  Gives reserved scratch space back to buffer (MG)
c     GETSCRS   Reserves a block of scratch space from buffer (MG)
c     RAPX      Forms coarse grid operator semicoarse in x-direction
c     RAPY      Forms coarse grid operator semicoarse in y-direction
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c   trivial
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      if (ibuf (corsnini).eq.SEMICORS) then

c--- Alternating RAPx and RAPy
         call RAPx (mat, rbuf ( ibuf( matri + 1)),
     +        ibuf (sizexi), ibuf (sizeyi),
     + ibuf(slacki),ibuf(slacki+1),ibuf(slacki+2),ibuf(slacki+3))
        do ilevel = 1, ibuf (hileveli) - 1
          if (mod (ilevel, 2).eq.1) then

c--- RAPy on odd levels (semicoarsening in y-direction)
             call RAPy (rbuf (ibuf (matri + ilevel)),
     +                  rbuf (ibuf (matri + ilevel + 1)),
     +       ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +     0,0,0,0)
          else

c--- RAPx on even levels (semicoarsening in x-direction)
             call RAPx (rbuf (ibuf (matri + ilevel)),
     +                  rbuf (ibuf (matri + ilevel + 1)),
     +           ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +     0,0,0,0)
          end if
        end do
      else

c     --- Both RAPx and RAPy on one level
c         First get some scratch space

      call getscrs ( ibuf ,
     +     ibuf (sizeyi)* ibuf (sizexi + 1) * 9)

        call RAPx (mat, rbuf ( ibuf (offscri)),
     +        ibuf (sizexi), ibuf (sizeyi),
     + ibuf(slacki),ibuf(slacki+1),ibuf(slacki+2),ibuf(slacki+3))
        call RAPy (rbuf (ibuf (offscri)),
     +             rbuf (ibuf (matri + 1)),
     +         ibuf (sizexi + 1), ibuf (sizeyi),
     +     0,0,0,0)

        do ilevel = 1, ibuf (hileveli) - 1
        call RAPx (rbuf ( ibuf (matri + ilevel)) ,
     +             rbuf ( ibuf (offscri)),
     +             ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +     0,0,0,0)

        call RAPy (rbuf ( ibuf (offscri)),
     +             rbuf ( ibuf (matri + ilevel + 1)),
     +             ibuf (sizexi + ilevel + 1), ibuf (sizeyi + ilevel),
     +     0,0,0,0)
        end do

c       --- free scratch space

      call freescrs (ibuf)
      end if
      end
