      subroutine prolonga ( finevec, coarsvec, nxf, nyf, nxc, nyc,
     +        rbuf, ibuf, sl1, sl2, sl3, sl4)
c ======================================================================
c
c        programmer     Jos van Kan
c
c       version 1.1    10-04-1997
c         utilizes module mgpreset instead of common block cmgconst
c       version 1.0    27-03-1997
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Prolongates  the vector according to the
c   the centering given by the parity of the fine grid.
c   Caters to all coarsenings
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c **********************************************************************
c
c         MODULES USED
c
      use mgpreset
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer sl1, sl2, sl3, sl4
      double precision finevec (*), coarsvec (*), rbuf(*)
      integer  nxf, nyf, nxc, nyc, ibuf (*)

c     coarsvec  i    Vector on coarse level
c     finevec   o    Vector on fine level
c     ibuf      i    Integer array of dimension 300 that contains the
c                    administration of this particular multi grid block.
c     nxc       i    Number of grid x points on coarse grid
c     nxf       i    Number of grid x points on fine grid
c     nyc       i    Number of grid y points on coarse grid
c     nyf       i    Number of grid y points on fine grid
c     rbuf      i    Buffer array used to store all coarse size info
c                    and as scratch space. The actaul bufferspace
c                    needed can be calculated using subr autosize.
c     sl1       i    Slack variable, x lower bound
c     sl2       i    Slack variable, x upper bound
c     sl3       i    Slack variable, y lower bound
c     sl4       i    Slack variable, y upper bound
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     FREESCRS  Gives reserved scratch space back to buffer (MG)
c     GETSCRS   Reserves a block of scratch space from buffer (MG)
c     PROLONX   Prolongation (semicoarsening) in x-direction (MG)
c     PROLONY   Prolongation (semicoarsening) in y-direction (MG)
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      if (nxf.eq.nxc) then

c     --- semicoarsening y-direction

        call prolony (finevec, coarsvec, nxf, nyf, sl1, sl2, sl3, sl4)
      else if (nyf.eq.nyc) then

c     --- semicoarsening x- direction

        call prolonx (finevec, coarsvec, nxf, nyf, sl1, sl2, sl3, sl4)
      else

c     --- full coarsening
c         --- get some scratch space

      call getscrs ( ibuf, nyf * nxc)

c     --- now prolongate first in y then in x direction. basically the order
c          is unimportant, but this is the way we implemented it

      call prolony ( rbuf (ibuf (offscri)), coarsvec, nxc, nyf,
     +     0,0,0,0)
      call prolonx ( finevec, rbuf (ibuf (offscri)), nxf, nyf, sl1, sl2,
     +     sl3, sl4)

c     --- free scratch space

      call freescrs (ibuf)
      end if
      end
